const config = require('../config.json');
const inquirer = require('inquirer');

const fsFcn = require('./fsFcn.js');
const dbFcn = require('./dbFcn.js');
const baseFcn = require('./baseFcn.js');

const to = require('./baseFcn.js').to;

var self = {
  // 使用者輸入
  cmdInput: function (prompt) {
    // return new Promise((resolve, reject) => {

    return new Promise((resolve) => {
      inquirer.prompt([prompt]).then((input) => {
        resolve(input)
      });
    })
  },

  // 顯示操作目錄
  showActionMenu: async function () {
    arg = await self.cmdInput({
      type: "input", name: "input",
      message: `請輸入對應數字，選擇功能：
  1. 取消操作
  2. 備份「來源資料庫」至「本機指定目錄」
  3. 載入「備份數據」寫入至「目標資料庫」
  4. 從「來源資料庫」複製數據至「目標資料庫」`,
      default: "1"
    })

    let ans = arg.input

    switch (ans) {
      case "1": {
        console.log("[ info ] 1. 取消操作");
        break;
      }

      case "2": {
        console.log("[ info ] 2. 備份「來源資料庫」至「本機指定目錄」");
        self.backupSourceDatas();
        break;
      }

      case "3": {
        console.log("[ info ] 3. 寫入備份數據至目標資料庫");
        self.writeBackupDatas();
        break;
      }

      case "4": {
        console.log("[ info ] 4. 從「來源資料庫」複製數據至「目標資料庫」");
        self.startCopyData();
        break;
      }

      default: {
        console.log("[ err ] 無效的輸入\n");
        self.showActionMenu();
      }
    }
  },

  // 備份「來源資料庫」至「本機指定目錄」
  backupSourceDatas: async function () {
    // 檢查 config 有效性
    if (!config.sourceDb && baseFcn.isEmpty(config.sourceDb)) {
      console.error("[ err ] : sourceDb 不可為空"); return;
    }
    if (!config.dbList && baseFcn.isEmpty(config.dbList)) {
      console.error("[ err ] : dbList 不可為空，請指定要備份的資料集"); return;
    }

    // 檢查路徑是否有效
    let path = config.backupDatasPath;
    if (path === fsFcn.defaultBackupPath()) {
      // 建立預設目錄
      if (!fsFcn.isValidPath(path))
        fsFcn.mkdir(fsFcn.defaultBackupPath());
      path = fsFcn.defaultBackupPath();

    } else if (!fsFcn.isValidPath(path)) {
      let arg = await self.cmdInput({
        type: "input", name: "input",
        message: `config 設定檔之備份目錄無效，是否要使用預設備份目錄？
                  ${fsFcn.defaultBackupPath()}`,
        default: "Y"
      })

      let ans = arg.input

      if (ans.toLowerCase() !== "y") {
        console.log("停止操作，請設定有效目錄"); return;
      }

      // 建立預設目錄
      fsFcn.mkdir(fsFcn.defaultBackupPath());
      path = fsFcn.defaultBackupPath();
    }

    // 顯示確認訊息
    console.log("\n[ info ] 準備開始備份 couchDB 資料\n");
    console.group(`[ info ] 來源資料庫`);
    console.log(`路徑     : ${config.sourceDb.url}`);
    console.log(`描述     : ${config.sourceDb.description}`);
    console.log(`授權帳號 : ${config.sourceDb.account}`);
    console.groupEnd(`[ info ] 來源資料庫`);

    console.log("");

    console.group(`[ info ] 備份清單`);
    console.log(`${config.dbList.join("、")}`);
    console.groupEnd(`[ info ] 備份清單`);

    console.log("");

    console.group(`[ info ] 儲存路徑`);
    console.log(`${path}`);
    console.groupEnd(`[ info ] 儲存路徑`);


    // 接收輸入
    let arg = await self.cmdInput({
      type: "input", name: "input",
      message: "請確認以上設定是否正確，是否開始備份？(Y/N)",
      default: "N"
    })

    if (arg.input.toLowerCase() !== "y") {
      console.log("[ info ] : 停止執行"); return;
    }

    // 開始執行所有備份任務
    let tasks = []
    try {
      for (let dbName of config.dbList) {
        let [err, result] = await to(dbFcn.backupDbAsFiles(dbName, path));
        tasks.push(err ? err : result);
      }
    } catch (err) {
      console.error("[ err ] : 發生意外錯誤 ", err);
    } finally {
      let sus = tasks.filter(result => result === "sus").length
      let err = tasks.filter(result => result === "err").length

      console.log(`\n[ info ] : 資料集備份結束，成功 ${sus} 個，失敗 ${err} 個，總共 ${sus + err} 個`);

      self.showActionMenu();
    }
  },

  // 將「備份資料」寫入至「目標資料庫」
  writeBackupDatas: async function () {
    // 檢查 config 有效性
    if (!config.targetDb && baseFcn.isEmpty(config.targetDb)) {
      console.error("[ err ] : targetDb 不可為空"); return;
    }

    // 檢查讀取路徑是否有效
    let path = config.sourceDatasPath;
    if (!fsFcn.isValidPath(path)) {
      console.error("[ err ] : 來源目錄無效，請設定來源目錄"); return;
    }

    // 讀取檔案
    let files = fsFcn.getLatestBackupFiles(path)
    if (files.length === 0) {
      console.error("[ err ] : 來源目錄沒有任何檔案，請放入備份檔案"); return;
    }


    // 顯示確認訊息
    console.log("\n[ info ] 準備開始寫入備份資料至 couchDB \n");
    console.group(`[ info ] 目標資料庫`);
    console.log(`路徑     : ${config.targetDb.url}`);
    console.log(`描述     : ${config.targetDb.description}`);
    console.log(`授權帳號 : ${config.targetDb.account}`);
    console.groupEnd(`[ info ] 目標資料庫`);

    console.log("");

    let arg = await self.cmdInput({
      type: "input", name: "input",
      message: `此動作會覆蓋「目標資料庫」同名資料，請仔細確認「目標資料庫」正確，以免清除原先資料，導致資料遺失。
                是否繼續？(Y/N)`,
      default: "N"
    })

    if (arg.input.toLowerCase() !== "y") {
      console.log("停止操作"); return;
    }

    console.group(`[ info ] 檔案來源目錄`);
    console.log(`${path}`);
    console.groupEnd(`[ info ] 檔案來源目錄`);

    console.log("");

    console.group(`[ info ] 檔案細節`);
    files.forEach((file) => {
      console.log(`資料集   : ${file.dbName}`);
      console.log(`檔名     : ${file.fileName}`);
      console.log(`備份時間 : ${baseFcn.formatStamp(file.timestamp)}`);
      console.log();
    })
    console.log(`以上共 ${files.length} 個資料集`);
    console.groupEnd(`[ info ] 檔案細節`);

    console.log("");

    arg = await self.cmdInput({
      type: "input", name: "input",
      message: `請確認資料集是否正確，是否開始寫入？(Y/N)`,
      default: "N"
    })

    if (arg.input.toLowerCase() !== "y") {
      console.log("停止操作"); return;
    }


    // 開始執行所有備份任務
    let tasks = []
    try {
      for (let file of files) {
        let [err, result] = await to(dbFcn.readBackupFilesWriteDb(file.dbName, file.fileName));
        tasks.push(err ? err : result);
      }
    } catch (err) {
      console.error("[ err ] : 發生意外錯誤 ", err);
    } finally {
      let sus = tasks.filter(result => result === "sus").length
      let err = tasks.filter(result => result === "err").length

      console.log(`\n[ info ] : 備份資料寫入結束，成功 ${sus} 個，失敗 ${err} 個，總共 ${sus + err} 個`);

      self.showActionMenu();
    }
  },

  // 從「來源資料庫」複製數據至「目標資料庫」
  startCopyData: async function () {
    // 檢查 config 有效性
    if (!config.sourceDb && baseFcn.isEmpty(config.sourceDb)) {
      console.error("[ err ] : sourceDb 不可為空"); return;
    }
    if (!config.targetDb && baseFcn.isEmpty(config.targetDb)) {
      console.error("[ err ] : targetDb 不可為空"); return;
    }
    if (!config.dbList && baseFcn.isEmpty(config.dbList)) {
      console.error("[ err ] : dbList 不可為空，請指定要複製的 database"); return;
    }

    // 顯示確認訊息
    console.log("\n[ info ] 準備開始複製 couchDB 資料\n");
    console.group(`來源資料庫`);
    console.log(`路徑     : ${config.sourceDb.url}`);
    console.log(`描述     : ${config.sourceDb.description}`);
    console.log(`授權帳號 : ${config.sourceDb.account}`);
    console.groupEnd(`來源資料庫`);

    console.log("");

    console.group(`目標資料庫`);
    console.log(`路徑     : ${config.targetDb.url}`);
    console.log(`描述     : ${config.targetDb.description}`);
    console.log(`授權帳號 : ${config.targetDb.account}`);
    console.groupEnd(`目標資料庫`);

    console.log("");

    console.group(`複製清單`);
    console.log(`${config.dbList.join("、")}`);
    console.groupEnd(`複製清單`);


    arg = await self.cmdInput({
      type: "input", name: "input",
      message: `請確認以上設定是否正確，是否開始複製？(Y/N)`,
      default: "N"
    })

    if (arg.input.toLowerCase() !== "y") {
      console.log("停止操作"); return;
    }

    //

    // 開始執行所有備份任務
    let tasks = []
    try {
      for (let dbName of config.dbList) {
        let [err, result] = await to(dbFcn.copyDatasToDb(dbName));
        tasks.push(err ? err : result);
      }
    } catch (err) {
      console.error("[ err ] : 發生意外錯誤 ", err);
    } finally {
      let sus = tasks.filter(result => result === "sus").length
      let err = tasks.filter(result => result === "err").length

      console.log(`\n[ info ] : 資料庫複製結束，成功 ${sus} 個，失敗 ${err} 個，總共 ${sus + err} 個`);

      self.showActionMenu();
    }
  },
}

module.exports = self