const config = require('../config.json');

const fsFcn = require("./fsFcn.js");

const to = require('./baseFcn.js').to;


var self = {
  // 生成資料庫連結，自動加入帳號
  createDbUrl: function (dbObj) {
    if (dbObj.account === "") {
      return dbObj.url
    }

    let urlSplit = dbObj.url.split("//")
    return `${urlSplit[0]}//${dbObj.account}:${dbObj.password}@${urlSplit[1]}`
  },

  // 複製指定資料集至目標資料庫
  copyDatasToDb: async function (dbName) {
    let sourceNano = require('nano')(self.createDbUrl(config.sourceDb));
    let targetNano = require('nano')(self.createDbUrl(config.targetDb));

    console.group(`[ info ] 開始複製 ${dbName} 資料`);

    try {
      // 檢查來源資料庫是否存在
      let sourceDbInfo = await self.dbGet(sourceNano, dbName, "sourceDbGet")
      console.log(`[ info ] 來源資料庫之 ${dbName} 資料集存在，檔案數 ${sourceDbInfo.doc_count} 個`);

      // 使用來源資料庫
      let sourceDb = sourceNano.use(dbName);

      // 取得來源資料庫所有資料
      let docs = await self.dbList(sourceDb)
      docs.forEach(doc => delete doc._rev); // 移除版本號
      console.log(`[ info ] 成功取得來源資料庫 ${dbName} 所有資料`);

      // 使用來源資料庫
      let targetDb = targetNano.use(dbName);

      // 直接刪除目標資料庫同名資料集
      await to(targetNano.db.destroy(dbName));
      await targetNano.db.create(dbName) // 目標資料庫創建資料集

      // 檢查目標資料庫是否存在
      await self.dbGet(targetNano, dbName, "targetDbGet")
      console.log(`[ info ] 目標資料庫 ${dbName} 連線成功，準備寫入`);

      // 寫入資料庫
      await targetDb.bulk({ docs })
      console.log(`[ info ] ${dbName} 資料寫入成功`);

      return Promise.resolve("sus");
    } catch (err) {
      console.log(`[ err ] 複製 ${dbName} 資料集發生錯誤`);

      console.log(`[ err ] ${err.msg}`);
      if (err.reason)
        console.log(`[ err ] ${err.reason}`);


      return Promise.reject("err");
    } finally {
      console.groupEnd(`[ info ] 開始複製 ${dbName} 資料`);
    }
  },

  // 備份資料集為檔案
  backupDbAsFiles: async function (dbName, path) {
    let sourceNano = require('nano')(self.createDbUrl(config.sourceDb));

    console.group(`[ info ] 開始備份 ${dbName}`);

    try {
      // 檢查來源資料庫是否存在
      let sourceDbInfo = await self.dbGet(sourceNano, dbName, "sourceDbGet")
      console.log(`[ info ] 來源資料庫之 ${dbName} 資料集存在，檔案數 ${sourceDbInfo.doc_count} 個`);

      // 使用來源資料庫
      let sourceDb = sourceNano.use(dbName);

      // 取得來源資料庫所有資料
      let docs = await self.dbList(sourceDb)
      console.log(`[ info ] 成功取得 ${dbName} 檔案，準備寫入檔案`);

      // 寫入檔案
      let fileName = fsFcn.fileName(dbName);
      await fsFcn.writeJson(path, fileName, docs)
      console.log(`[ info ] 生成 ${fileName}，成功備份 ${dbName} 資料`);


      return Promise.resolve("sus");
    } catch (err) {
      console.log(`[ err ] 備份 ${dbName} 資料集發生錯誤`);

      console.log(`[ err ] ${err.msg}`);
      if (err.reason)
        console.log(`[ err ] ${err.reason}`);


      return Promise.reject("err");
    } finally {
      console.groupEnd(`[ info ] 開始備份 ${dbName}`);
    }
  },

  // 從備份目錄讀取檔案，並寫入目標資料庫
  readBackupFilesWriteDb: async function (dbName, fileName) {
    let targetNano = require('nano')(self.createDbUrl(config.targetDb));

    console.group(`[ info ] 開始寫入 ${dbName} 備份資料`);

    try {
      // 直接刪除目標資料庫同名資料集
      await to(targetNano.db.destroy(dbName));
      await targetNano.db.create(dbName) // 目標資料庫創建資料集

      // 檢查目標資料庫是否存在
      await self.dbGet(targetNano, dbName, "targetDbGet")
      console.log(`[ info ] 目標資料庫 ${dbName} 連線成功`);

      // 使用來源資料庫
      let targetDb = targetNano.use(dbName);

      // 載入備份資料
      let path = config.sourceDatasPath;
      let { data: backupData, size: fileSize } = await fsFcn.readJson(path, fileName, "readBackupData")
      console.log(`[ info ] 載入 ${dbName} 備份資料成功，檔案大小 ${(fileSize / 1000).toFixed(2)} kB`);

      // 寫入資料庫
      await targetDb.bulk({ docs: backupData })
      console.log(`[ info ] ${dbName} 備份資料寫入成功`);

      return Promise.resolve("sus");
    } catch (err) {
      console.log(`[ err ] 寫入 ${dbName} 資料集發生錯誤`);

      console.log(`[ err ] ${err.msg}`);
      if (err.reason)
        console.log(`[ err ] ${err.reason}`);


      return Promise.reject("err");
    } finally {
      console.groupEnd(`[ info ] 開始寫入 ${dbName} 備份資料`);
    }
  },

  // 取得資料庫訊息
  dbGet: function (nano, dbName, step = "") {
    return new Promise((resolve, reject) => {
      // 確認來源資料庫是否存在
      nano.db.get(dbName).then((body) => {
        resolve(body);
      }).catch((err) => {
        switch (err.statusCode) {
          case 401: {
            reject({
              step,
              status: err.statusCode,
              msg: `帳號密碼錯誤，請檢查帳號密碼是否正確`,
            })
            break;
          }

          case 404: {
            reject({
              step,
              status: err.statusCode,
              msg: `${dbName} 資料集不存在，請確認名稱正確性`,
            })
            break;
          }

          default: {
            reject({
              step,
              status: err.statusCode,
              msg: "發生意外錯誤",
              reason: err.reason
            })
          }
        }

      })
    })
  },
  // 取得全部 doc
  dbList: function (db, step = "") {
    return new Promise((resolve, reject) => {
      db.list({ include_docs: true }).then((body) => {
        resolve(body.rows.map((doc) => {
          let val = doc.doc;
          delete val._rev;   // 移除版本紀錄

          return val
        }));
      }).catch((err) => {
        reject({
          step,
          status: err.statusCode,
          msg: "發生意外錯誤",
          reason: err.reason
        })
      })
    })
  },
}

module.exports = self